#include <atk/atkeditabletext.h>
#include <stdlib.h>
#include "testtextlib.h"

#define NUM_VALID_ROLES 5  

static void _create_event_watcher ();
static void _check_text (AtkObject *obj);
void runtest(AtkObject *, gint);

static guint id1 = 0;
static guint win_count = 0;

static void _check_text (AtkObject *in_obj)
{
  AtkObject *obj = 0;
  AtkRole role;
  gchar* title;
  AtkRole valid_roles[NUM_VALID_ROLES];

  if (getenv("TEST_ACCESSIBLE_DELAY") != NULL)
  {
    int max_cnt = string_to_int(getenv("TEST_ACCESSIBLE_DELAY"));
    win_count++;
    if (win_count <= max_cnt)
      return;
  }

  /* Set Up */

  valid_roles[0] = ATK_ROLE_TEXT;
  valid_roles[1] = ATK_ROLE_LABEL;
  valid_roles[2] = ATK_ROLE_ACCEL_LABEL;
  valid_roles[3] = ATK_ROLE_PASSWORD_TEXT;
  valid_roles[4] = ATK_ROLE_TABLE_CELL;
  
  /* The following if/else grabs the windows name, or sets title to NULL if none. */
  if (in_obj->name)
  {
     title = in_obj->name;
  }
  else
  {
    GtkWidget* widget = GTK_ACCESSIBLE (in_obj)->widget;

    if (widget == NULL)
    {
      title = NULL;
    }

    if (GTK_IS_WINDOW (widget) && GTK_WINDOW (widget)->title)
    {
      title = GTK_WINDOW (widget)->title;
    }
    else
      title = NULL;
  }
  /* If no window name, do nothing */
  if (title == NULL) 
    return;
  /* If testtext test program, find obj just by role since only one child with no name */ 
  else if (g_strncasecmp(title, "testtext", 7) == 0) 
  {
      obj = find_object_by_role(in_obj, valid_roles, NUM_VALID_ROLES);
    if (obj != NULL)
       g_print("Found valid role in child!\n");
  }
  /* Otherwise, get obj by name and role so you can specify exactly which obj to run tests on */
  else 
  {
    if (getenv("TEST_ACCESSIBLE_NAME") != NULL)
    {
      obj = find_object_by_name_and_role(in_obj,
        getenv("TEST_ACCESSIBLE_NAME"), valid_roles, NUM_VALID_ROLES);
    }
    if (obj != NULL)
       g_print("Found valid name and role in child!\n");
    else
    {
       obj = find_object_by_role(in_obj, valid_roles, NUM_VALID_ROLES);
       if (obj != NULL)
          g_print("Found valid role in child!\n");
    }   
  } 
  if (obj == NULL)
  {
     g_print("Object not found\n");
     return;
  }

  role = atk_object_get_role(obj);

  g_print ("In _check_text\n");
  g_print("_check_text - Found role type %s!\n\n", get_role_name(role));

  add_handlers(obj);

  if (!(isVisibleDialog()))
    setup_gui(obj, runtest);
}

static void
_create_event_watcher ()
{
  id1 = atk_add_focus_tracker (_check_text);
}

int
gtk_module_init(gint argc, char* argv[])
{
  g_print("testtext Module loaded.\n");
  _create_event_watcher();

  return 0;
}


